/* $Id: startup-lsb.c,v 1.2 2005/06/26 06:17:36 short Exp $
 * UDP Gateway utility startup scripts support using lsb(8)
 * Copyright (C) 2004 Jan Kratochvil <project-udpgate@jankratochvil.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; exactly version 2 of June 1991 is required
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "config.h"

#include <glib/gmessages.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <glib/gmem.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include "startup-lsb.h"	/* self */

#ifdef ENABLE_BUNDLE
#include "bundle-util.h"
#endif


/* Config: */
/* Do not: /etc/rc.d/init.d/...
 * to comply with http://www.linuxbase.org/spec/booksets/LSB-Core/LSB-Core.html#INITSRCINSTRM
 */
#define INIT_D_PATHNAME G_STRINGIFY(SYSCONFDIR) "/init.d/" PACKAGE

#define COMMAND_INSTALL_INIT_D "/usr/lib/lsb/install_initd"
#define COMMAND_REMOVE_INIT_D  "/usr/lib/lsb/remove_initd"


#define STATUS_0_1(status) ( \
		!(WIFEXITED((status)) && (WEXITSTATUS((status))==0 || WEXITSTATUS((status))==1) \
						&& !WIFSIGNALED((status)) \
						&& !WIFSTOPPED((status))) \
				? -1 : WEXITSTATUS((status)))

static gboolean chkexec(const gchar *binary)
{
struct stat statbuf;

	g_return_val_if_fail(binary!=NULL,FALSE);

	if (stat(COMMAND_INSTALL_INIT_D,&statbuf))
		return FALSE;
	if ((statbuf.st_mode&(S_IFMT|0111))!=(S_IFREG|0111))
		return FALSE;

	return TRUE;
}


static gboolean udpgate_startup_lsb_init(UdpgateStartup *udpgate_startup)
{
	g_return_val_if_fail(UDPGATE_IS_STARTUP_LSB(udpgate_startup),FALSE);

	/* LSB compliant 'query' is not standardized. */
	if (chkexec(COMMAND_INSTALL_INIT_D) && chkexec(COMMAND_REMOVE_INIT_D))
		return TRUE;
	return FALSE;
}

static gboolean udpgate_startup_lsb_query(UdpgateStartup *udpgate_startup,gboolean *is_on)
{
	g_return_val_if_fail(UDPGATE_IS_STARTUP_LSB(udpgate_startup),FALSE);

	/* LSB compliant 'query' is not standardized. */
	return FALSE;
}

static gboolean udpgate_startup_lsb_on(UdpgateStartup *udpgate_startup)
{
const gchar *command=COMMAND_INSTALL_INIT_D " " PACKAGE;
int status;

	g_return_val_if_fail(UDPGATE_IS_STARTUP_LSB(udpgate_startup),FALSE);

#ifdef ENABLE_BUNDLE
	if (!bundle_util_file_write(
			INIT_D_PATHNAME,	/* pathname */
			PACKAGE ".init",	/* basename */
			0755,	/* pathname_mode */
			BUNDLE_UTIL_BACKUP_MASK))	/* flags */
		return FALSE;
#endif /* ENABLE_BUNDLE */

	status=system(command);
	if (0!=STATUS_0_1(status)) {
		g_warning(_("Error registering automatic program startup by: %s"),command);
		return FALSE;
		}
	return TRUE;
}

static gboolean udpgate_startup_lsb_off(UdpgateStartup *udpgate_startup)
{
const gchar *command=COMMAND_REMOVE_INIT_D " " PACKAGE;
int status;

	g_return_val_if_fail(UDPGATE_IS_STARTUP_LSB(udpgate_startup),FALSE);

	status=system(command);
	if (0!=STATUS_0_1(status)) {
		g_warning(_("Error removing program's system startup registrance by: %s"),command);
		return FALSE;
		}

#ifdef ENABLE_BUNDLE
	if (!bundle_util_file_remove(INIT_D_PATHNAME,PACKAGE ".init"))
		return FALSE;
#endif /* ENABLE_BUNDLE */

	return TRUE;
}

static void udpgate_startup_lsb_class_init (UdpgateStartupLsbClass *class)
{
	UdpgateStartupClass *udpgate_startup_class = UDPGATE_STARTUP_CLASS(class);

	udpgate_startup_class->init = udpgate_startup_lsb_init;
	udpgate_startup_class->query = udpgate_startup_lsb_query;
	udpgate_startup_class->on = udpgate_startup_lsb_on;
	udpgate_startup_class->off = udpgate_startup_lsb_off;
}

GType udpgate_startup_lsb_get_type(void)
{
  static GType udpgate_startup_lsb_type=0;

  if (!udpgate_startup_lsb_type) {
		static const GTypeInfo udpgate_startup_lsb_info={
			sizeof(UdpgateStartupLsbClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc)udpgate_startup_lsb_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof(UdpgateStartupLsb),
			0,		/* n_preallocs */
			NULL,		/* instance_init */
			NULL,		/* value_table */
      };

		udpgate_startup_lsb_type=g_type_register_static(UDPGATE_TYPE_STARTUP,"UdpgateStartupLsb",
				 &udpgate_startup_lsb_info,0);
    }

  return udpgate_startup_lsb_type;
}
